package org.example.SnakeGame;

import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.text.Text;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.example.game.Direction;

import java.net.URL;
import java.util.Random;
import java.util.ResourceBundle;

public class SnakeGameController implements Initializable {

    private final static Logger log = LogManager.getLogger(SnakeGameController.class);
    private static final int radius = 14;

    private static final int width = 600;
    private static final int height = 500;
    @FXML
    private AnchorPane anchorPane;


    //food
    private Circle food;
    private Text score;
    private Random random;
    private Snake snake;
    // private Direction currentDirection;

    /**
     * methode will create the food
     * and display it randomly
     */
    private void newFood(){
    log.info("Food was created");
    food = new Circle(random.nextInt(400),random.nextInt(500), radius);
    food.setFill(Color.DARKGREEN);
    anchorPane.getChildren().add(food);
    }

    /**
     * methode will create
     * the snake
     */
    private void newSnake(){
        log.info("Snake was created");
        snake = new Snake(300,250, radius +2);
        anchorPane.getChildren().add(snake);
        for (int i = 0; i < 25; i++){
            snake.eat(food);
        }
    }

    private boolean hit(){
        return food.intersects(snake.getBoundsInLocal());
    }
    private boolean gameOver(){
        return snake.eatSelf();
    }
    private void move(){
        Platform.runLater(() ->{
            snake.step();
            adjustLocation();
            if (hit() ){
                log.info("Collected food");
                snake.eat(food);
                score.setText("Score: "+ snake.getLength());
                newFood();
                log.info("Spawned new food");

            }
            else if (gameOver()){
                log.info("Game Over");
                anchorPane.getChildren().clear();
                anchorPane.getChildren().add(score);
                score.setText("Game Over "+ snake.getLength());
            }
        });
    }

    /**
     * methode will adjust the location
     * of the snake and places it at the other side,
     * if it moves out of the window
     */
    private void adjustLocation(){
        if (snake.getCenterX()<0){
            snake.setCenterX(width);
        }
        else if (snake.getCenterX()>width){
            snake.setCenterX(0);
        }
        if (snake.getCenterY()<0){
            snake.setCenterY(height);
        }
        else if (snake.getCenterY()>height){
            snake.setCenterY(0);
        }
    }

    @Override
    public void initialize(URL location, ResourceBundle resources) {

        random = new Random();
        score = new Text(0, 33 , "0");

        newFood();
        newSnake();

        /**
         * changes speed of snake
         * after eating food
         */

        Runnable r = ()->{
            try {
                for (; ; ) {
                    move();
                    Thread.sleep(70/((snake.getLength() / 10)));
                }
            }catch(InterruptedException ignored){
                log.warn("Illegal input");
            }};


        anchorPane.sceneProperty().addListener((observable, oldScene, newScene) -> {
            if (newScene != null) {
                newScene.setOnKeyPressed(event -> {
                    KeyCode code = event.getCode();

                    if (code == KeyCode.UP){
                        snake.setCurrentDirection(Direction.UP);
                        log.info("User input UP");
                    }
                    else if (code == KeyCode.DOWN){
                        snake.setCurrentDirection(Direction.DOWN);
                        log.info("User input DOWN");
                    }
                    else if (code == KeyCode.LEFT) {
                        snake.setCurrentDirection(Direction.LEFT);
                        log.info("User input LEFT");
                    }
                    else if (code == KeyCode.RIGHT) {
                        snake.setCurrentDirection(Direction.RIGHT);
                        log.info("User input RIGHT");
                    }



                });
            }
        });

        Thread th = new Thread(r);
        th.setDaemon(true);
        th.start();



    }
}